/***************************************************************************
 *   Copyright (C) 2006-2011 by Julien Lemoine, Simon Viennot              *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "splitboard.h"

#include <QString>

//return true if the position is composed of multiple boards
bool SplitBoard::isMultiComponents() {
	return (boardList.size() > (unsigned int) 1);
}

//return the list of components as individual positions
list<Line> SplitBoard::sumComponents() {
	list<Line> result;
	
	//we need to transform each string in a Line
	int index;
	for(index = 0; index < (int) boardList.size(); index++) {
		//initialize the current board
		initCurrentBoard(index);
		result.push_back(toLine());
	}
	
	return result;
}

//---------------------- Algorithm to separate independent components------------------------------------------

//recursively color one connected component of the board
void SplitBoard::colorOneComponent(int i0, int j0, int colorNumber) {
	//nothing to do if the cell is already colored
	if(colors[i0*columns+j0]>=0) return;
	
	//color the current cell
	colors[i0*columns+j0]=colorNumber;
	
	//color recursively all the adjacent cells
	int direction;
	for(direction=0; direction<4; direction++) {
		int i=i0, j=j0;
		if(next(i, j, direction)) {
			//this is a valid direction, try to extend the connex component
			colorOneComponent(i, j, colorNumber);
		}
	}
}

//color the connected components of the board, with a different color for each component
//return the number of colors, which is also the number of components
int SplitBoard::colorComponents() {
	colors.clear();
	
	//color 0 means an occupied gray square, color -1 means any other square (considered empty)
	int i, j;
	for(i=0; i<rows; i++) {
		for(j=0; j<columns; j++) {
			if(getCell(i,j) != Gray) {
				colors.push_back(-1);
			} else {
				colors.push_back(0);
			}
		}
	}
	
	int colorNumber = 0;
	
	//color the board until all cell are colored
	for(i=0; i<rows; i++) {
		for(j=0; j<columns; j++) {
			if(colors[i*columns + j]<0) {
				colorNumber++;
				colorOneComponent(i, j, colorNumber);
			}
		}
	}
	
	return colorNumber;
}

//Obtain the board corresponding to a given color (=one connected component)
//Cells different from the given color are filled with Gray
void SplitBoard::getOneComponent(int colorNumber) {
	int i, j;
	for(i=0; i<rows; i++) {
		for(j=0; j<columns; j++) {
			if(colors[i*columns + j]!=colorNumber) {
				setCell(i, j, Gray);
			}
		}
	}
}

//---------------------- Canonization -----------------------------------------

//by default, canonize with all height symmetries (vertical, horizontal, rotations)
int SplitBoard::symmetryForCano() {
	return BoardTable::EightSymmetries;
}

//canonization of a board list
//separate each board into independent components
void SplitBoard::canonize() {
	if(boardList.empty()) return;

	vector<string> result;
	
	//we need to separate into components each board of the board list
	int index;
	for(index = 0; index < (int) boardList.size(); index++) {
		//initialize the current board
		initCurrentBoard(index);
		int colorNumber = colorComponents();
		saveBoard(); //getOneComponent changes the board, so we save it
		
		int i;
		for(i=1; i<=colorNumber; i++) {
			getOneComponent(i);
			reduceSize();
			symmetry(symmetryForCano());  //canonize the authorized symetries
			result.push_back(toString());
			
			//caution : we need to go back to the initial board, in order to get correctly the next component
			backToSavedBoard();
		}
	}
	
	//update the boardList with the result list
	boardList = result;
	
	//initialize the current board to the first board of the list
	if(!boardList.empty()) initCurrentBoard(0);
}

//! Reduce the size of the board, by deleting all the surrounding filled rows or columns 
void SplitBoard::reduceSize() {
	//first step : find the smallest containing rectangle between (i0, j0) -> (i1,j1)
	//during the algorithm, (i0, j0) will only decrease, and (i1, j1) will only increase
	int i0=rows-1, j0=columns-1, i1=0, j1=0;
	int i, j;
	for(i=0; i<rows; i++) {
		for(j=0; j<columns; j++) {
			if(getCell(i, j) != Gray ) {
				//a non-gray cell gives new conditions on the smallest rectangle
				if(i<i0) i0 = i;
				if(j<j0) j0 = j;
				if(i>i1) i1 = i;
				if(j>j1) j1 = j;
			}
		}
	}
	
	//second step : reduce the size of the board to the rectangle i0, j0, i1, j1
	reduceBoardTo(i0, j0, i1, j1);
}
